@@ -1,14 +1,15 @@
 """Tests for certbot.plugins.manual"""
-import os
 import unittest
+import sys
 
-import six
 import mock
+import six
 
 from acme import challenges
 
 from certbot import errors
-
+from certbot.compat import os
+from certbot.compat import filesystem
 from certbot.tests import acme_util
 from certbot.tests import util as test_util
 
@@ -20,10 +21,10 @@ class AuthenticatorTest(test_util.TempDirTestCase):
         super(AuthenticatorTest, self).setUp()
         self.http_achall = acme_util.HTTP01_A
         self.dns_achall = acme_util.DNS01_A
-        self.tls_sni_achall = acme_util.TLSSNI01_A
-        self.achalls = [self.http_achall, self.dns_achall, self.tls_sni_achall]
+        self.dns_achall_2 = acme_util.DNS01_A_2
+        self.achalls = [self.http_achall, self.dns_achall, self.dns_achall_2]
         for d in ["config_dir", "work_dir", "in_progress"]:
-            os.mkdir(os.path.join(self.tempdir, d))
+            filesystem.mkdir(os.path.join(self.tempdir, d))
             # "backup_dir" and "temp_checkpoint_dir" get created in
             # certbot.util.make_or_verify_dir() during the Reverter
             # initialization.
@@ -36,8 +37,7 @@ class AuthenticatorTest(test_util.TempDirTestCase):
             backup_dir=os.path.join(self.tempdir, "backup_dir"),
             temp_checkpoint_dir=os.path.join(
                                         self.tempdir, "temp_checkpoint_dir"),
-            in_progress_dir=os.path.join(self.tempdir, "in_progess"),
-            tls_sni_01_port=5001)
+            in_progress_dir=os.path.join(self.tempdir, "in_progess"))
 
         from certbot.plugins.manual import Authenticator
         self.auth = Authenticator(self.config, name='manual')
@@ -56,9 +56,7 @@ class AuthenticatorTest(test_util.TempDirTestCase):
 
     def test_get_chall_pref(self):
         self.assertEqual(self.auth.get_chall_pref('example.org'),
-                         [challenges.HTTP01,
-                          challenges.DNS01,
-                          challenges.TLSSNI01])
+                         [challenges.HTTP01, challenges.DNS01])
 
     @test_util.patch_get_utility()
     def test_ip_logging_not_ok(self, mock_get_utility):
@@ -74,19 +72,16 @@ class AuthenticatorTest(test_util.TempDirTestCase):
     def test_script_perform(self):
         self.config.manual_public_ip_logging_ok = True
         self.config.manual_auth_hook = (
-            'echo ${CERTBOT_DOMAIN}; '
-            'echo ${CERTBOT_TOKEN:-notoken}; '
-            'echo ${CERTBOT_CERT_PATH:-nocert}; '
-            'echo ${CERTBOT_KEY_PATH:-nokey}; '
-            'echo ${CERTBOT_SNI_DOMAIN:-nosnidomain}; '
-            'echo ${CERTBOT_VALIDATION:-novalidation};')
-        dns_expected = '{0}\n{1}\n{2}\n{3}\n{4}\n{5}'.format(
+            '{0} -c "from __future__ import print_function;'
+            'from certbot.compat import os;  print(os.environ.get(\'CERTBOT_DOMAIN\'));'
+            'print(os.environ.get(\'CERTBOT_TOKEN\', \'notoken\'));'
+            'print(os.environ.get(\'CERTBOT_VALIDATION\', \'novalidation\'));"'
+            .format(sys.executable))
+        dns_expected = '{0}\n{1}\n{2}'.format(
             self.dns_achall.domain, 'notoken',
-            'nocert', 'nokey', 'nosnidomain',
             self.dns_achall.validation(self.dns_achall.account_key))
-        http_expected = '{0}\n{1}\n{2}\n{3}\n{4}\n{5}'.format(
+        http_expected = '{0}\n{1}\n{2}'.format(
             self.http_achall.domain, self.http_achall.chall.encode('token'),
-            'nocert', 'nokey', 'nosnidomain',
             self.http_achall.validation(self.http_achall.account_key))
 
         self.assertEqual(
@@ -98,17 +93,6 @@ class AuthenticatorTest(test_util.TempDirTestCase):
         self.assertEqual(
             self.auth.env[self.http_achall]['CERTBOT_AUTH_OUTPUT'],
             http_expected)
-        # tls_sni_01 challenge must be perform()ed above before we can
-        # get the cert_path and key_path.
-        tls_sni_expected = '{0}\n{1}\n{2}\n{3}\n{4}\n{5}'.format(
-            self.tls_sni_achall.domain, 'notoken',
-            self.auth.tls_sni_01.get_cert_path(self.tls_sni_achall),
-            self.auth.tls_sni_01.get_key_path(self.tls_sni_achall),
-            self.auth.tls_sni_01.get_z_domain(self.tls_sni_achall),
-            'novalidation')
-        self.assertEqual(
-            self.auth.env[self.tls_sni_achall]['CERTBOT_AUTH_OUTPUT'],
-            tls_sni_expected)
 
     @test_util.patch_get_utility()
     def test_manual_perform(self, mock_get_utility):
@@ -118,18 +102,14 @@ class AuthenticatorTest(test_util.TempDirTestCase):
             [achall.response(achall.account_key) for achall in self.achalls])
         for i, (args, kwargs) in enumerate(mock_get_utility().notification.call_args_list):
             achall = self.achalls[i]
-            if isinstance(achall.chall, challenges.TLSSNI01):
-                self.assertTrue(
-                        self.auth.tls_sni_01.get_cert_path(
-                            self.tls_sni_achall) in args[0])
-            else:
-                self.assertTrue(
-                        achall.validation(achall.account_key) in args[0])
+            self.assertTrue(
+                achall.validation(achall.account_key) in args[0])
             self.assertFalse(kwargs['wrap'])
 
     def test_cleanup(self):
         self.config.manual_public_ip_logging_ok = True
-        self.config.manual_auth_hook = 'echo foo;'
+        self.config.manual_auth_hook = ('{0} -c "import sys; sys.stdout.write(\'foo\')"'
+                                        .format(sys.executable))
         self.config.manual_cleanup_hook = '# cleanup'
         self.auth.perform(self.achalls)
 
@@ -137,8 +117,7 @@ class AuthenticatorTest(test_util.TempDirTestCase):
             self.auth.cleanup([achall])
             self.assertEqual(os.environ['CERTBOT_AUTH_OUTPUT'], 'foo')
             self.assertEqual(os.environ['CERTBOT_DOMAIN'], achall.domain)
-            if (isinstance(achall.chall, challenges.HTTP01) or
-                isinstance(achall.chall, challenges.DNS01)):
+            if isinstance(achall.chall, (challenges.HTTP01, challenges.DNS01)):
                 self.assertEqual(
                     os.environ['CERTBOT_VALIDATION'],
                     achall.validation(achall.account_key))
@@ -148,18 +127,6 @@ class AuthenticatorTest(test_util.TempDirTestCase):
                     achall.chall.encode('token'))
             else:
                 self.assertFalse('CERTBOT_TOKEN' in os.environ)
-            if isinstance(achall.chall, challenges.TLSSNI01):
-                self.assertEqual(
-                    os.environ['CERTBOT_CERT_PATH'],
-                    self.auth.tls_sni_01.get_cert_path(achall))
-                self.assertEqual(
-                    os.environ['CERTBOT_KEY_PATH'],
-                    self.auth.tls_sni_01.get_key_path(achall))
-                self.assertFalse(
-                    os.path.exists(os.environ['CERTBOT_CERT_PATH']))
-                self.assertFalse(
-                    os.path.exists(os.environ['CERTBOT_KEY_PATH']))
-
 
 
 if __name__ == '__main__':
