@@ -1,17 +1,17 @@
 """Tests for certbot.util."""
 import argparse
 import errno
-import os
-import shutil
+import sys
 import unittest
 
 import mock
 import six
 from six.moves import reload_module  # pylint: disable=import-error
 
-from certbot import compat
-from certbot import errors
 import certbot.tests.util as test_util
+from certbot import errors
+from certbot.compat import os
+from certbot.compat import filesystem
 
 
 class RunScriptTest(unittest.TestCase):
@@ -53,26 +53,13 @@ class ExeExistsTest(unittest.TestCase):
         from certbot.util import exe_exists
         return exe_exists(exe)
 
-    @mock.patch("certbot.util.os.path.isfile")
-    @mock.patch("certbot.util.os.access")
-    def test_full_path(self, mock_access, mock_isfile):
-        mock_access.return_value = True
-        mock_isfile.return_value = True
-        self.assertTrue(self._call("/path/to/exe"))
+    def test_exe_exists(self):
+        with mock.patch("certbot.util.filesystem.is_executable", return_value=True):
+            self.assertTrue(self._call("/path/to/exe"))
 
-    @mock.patch("certbot.util.os.path.isfile")
-    @mock.patch("certbot.util.os.access")
-    def test_on_path(self, mock_access, mock_isfile):
-        mock_access.return_value = True
-        mock_isfile.return_value = True
-        self.assertTrue(self._call("exe"))
-
-    @mock.patch("certbot.util.os.path.isfile")
-    @mock.patch("certbot.util.os.access")
-    def test_not_found(self, mock_access, mock_isfile):
-        mock_access.return_value = False
-        mock_isfile.return_value = True
-        self.assertFalse(self._call("exe"))
+    def test_exe_not_exists(self):
+        with mock.patch("certbot.util.filesystem.is_executable", return_value=False):
+            self.assertFalse(self._call("/path/to/exe"))
 
 
 class LockDirUntilExit(test_util.TempDirTestCase):
@@ -88,23 +75,26 @@ class LockDirUntilExit(test_util.TempDirTestCase):
         import certbot.util
         reload_module(certbot.util)
 
-    @test_util.broken_on_windows
     @mock.patch('certbot.util.logger')
     @mock.patch('certbot.util.atexit_register')
     def test_it(self, mock_register, mock_logger):
         subdir = os.path.join(self.tempdir, 'subdir')
-        os.mkdir(subdir)
+        filesystem.mkdir(subdir)
         self._call(self.tempdir)
         self._call(subdir)
         self._call(subdir)
 
         self.assertEqual(mock_register.call_count, 1)
         registered_func = mock_register.call_args[0][0]
-        shutil.rmtree(subdir)
-        registered_func()  # exception not raised
-        # logger.debug is only called once because the second call
-        # to lock subdir was ignored because it was already locked
-        self.assertEqual(mock_logger.debug.call_count, 1)
+
+        from certbot import util
+        # Despite lock_dir_until_exit has been called twice to subdir, its lock should have been
+        # added only once. So we expect to have two lock references: for self.tempdir and subdir
+        self.assertTrue(len(util._LOCKS) == 2)  # pylint: disable=protected-access
+        registered_func()  # Exception should not be raised
+        # Logically, logger.debug, that would be invoked in case of unlock failure,
+        # should never been called.
+        self.assertEqual(mock_logger.debug.call_count, 0)
 
 
 class SetUpCoreDirTest(test_util.TempDirTestCase):
@@ -117,15 +107,14 @@ class SetUpCoreDirTest(test_util.TempDirTestCase):
     @mock.patch('certbot.util.lock_dir_until_exit')
     def test_success(self, mock_lock):
         new_dir = os.path.join(self.tempdir, 'new')
-        self._call(new_dir, 0o700, compat.os_geteuid(), False)
+        self._call(new_dir, 0o700, False)
         self.assertTrue(os.path.exists(new_dir))
         self.assertEqual(mock_lock.call_count, 1)
 
     @mock.patch('certbot.util.make_or_verify_dir')
     def test_failure(self, mock_make_or_verify):
         mock_make_or_verify.side_effect = OSError
-        self.assertRaises(errors.Error, self._call,
-                          self.tempdir, 0o700, compat.os_geteuid(), False)
+        self.assertRaises(errors.Error, self._call, self.tempdir, 0o700, False)
 
 
 class MakeOrVerifyDirTest(test_util.TempDirTestCase):
@@ -140,60 +129,31 @@ class MakeOrVerifyDirTest(test_util.TempDirTestCase):
         super(MakeOrVerifyDirTest, self).setUp()
 
         self.path = os.path.join(self.tempdir, "foo")
-        os.mkdir(self.path, 0o600)
-
-        self.uid = compat.os_geteuid()
+        filesystem.mkdir(self.path, 0o600)
 
     def _call(self, directory, mode):
         from certbot.util import make_or_verify_dir
-        return make_or_verify_dir(directory, mode, self.uid, strict=True)
+        return make_or_verify_dir(directory, mode, strict=True)
 
     def test_creates_dir_when_missing(self):
         path = os.path.join(self.tempdir, "bar")
         self._call(path, 0o650)
         self.assertTrue(os.path.isdir(path))
-        self.assertTrue(compat.compare_file_modes(os.stat(path).st_mode, 0o650))
+        self.assertTrue(filesystem.check_mode(path, 0o650))
 
     def test_existing_correct_mode_does_not_fail(self):
         self._call(self.path, 0o600)
-        self.assertTrue(compat.compare_file_modes(os.stat(self.path).st_mode, 0o600))
+        self.assertTrue(filesystem.check_mode(self.path, 0o600))
 
-    @test_util.skip_on_windows('Umask modes are mostly ignored on Windows.')
     def test_existing_wrong_mode_fails(self):
         self.assertRaises(errors.Error, self._call, self.path, 0o400)
 
     def test_reraises_os_error(self):
-        with mock.patch.object(os, "makedirs") as makedirs:
+        with mock.patch.object(filesystem, "makedirs") as makedirs:
             makedirs.side_effect = OSError()
             self.assertRaises(OSError, self._call, "bar", 12312312)
 
 
-class CheckPermissionsTest(test_util.TempDirTestCase):
-    """Tests for certbot.util.check_permissions.
-
-    Note that it is not possible to test for a wrong file owner,
-    as this testing script would have to be run as root.
-
-    """
-
-    def setUp(self):
-        super(CheckPermissionsTest, self).setUp()
-
-        self.uid = compat.os_geteuid()
-
-    def _call(self, mode):
-        from certbot.util import check_permissions
-        return check_permissions(self.tempdir, mode, self.uid)
-
-    def test_ok_mode(self):
-        os.chmod(self.tempdir, 0o600)
-        self.assertTrue(self._call(0o600))
-
-    def test_wrong_mode(self):
-        os.chmod(self.tempdir, 0o400)
-        self.assertFalse(self._call(0o600))
-
-
 class UniqueFileTest(test_util.TempDirTestCase):
     """Tests for certbot.util.unique_file."""
 
@@ -210,16 +170,21 @@ class UniqueFileTest(test_util.TempDirTestCase):
         fd, name = self._call()
         fd.write("bar")
         fd.close()
-        self.assertEqual(open(name).read(), "bar")
+        with open(name) as f:
+            self.assertEqual(f.read(), "bar")
 
     def test_right_mode(self):
-        self.assertTrue(compat.compare_file_modes(0o700, os.stat(self._call(0o700)[1]).st_mode))
-        self.assertTrue(compat.compare_file_modes(0o600, os.stat(self._call(0o600)[1]).st_mode))
+        fd1, name1 = self._call(0o700)
+        fd2, name2 = self._call(0o600)
+        self.assertTrue(filesystem.check_mode(name1, 0o700))
+        self.assertTrue(filesystem.check_mode(name2, 0o600))
+        fd1.close()
+        fd2.close()
 
     def test_default_exists(self):
-        name1 = self._call()[1]  # create 0000_foo.txt
-        name2 = self._call()[1]
-        name3 = self._call()[1]
+        fd1, name1 = self._call()  # create 0000_foo.txt
+        fd2, name2 = self._call()
+        fd3, name3 = self._call()
 
         self.assertNotEqual(name1, name2)
         self.assertNotEqual(name1, name3)
@@ -236,6 +201,10 @@ class UniqueFileTest(test_util.TempDirTestCase):
         basename3 = os.path.basename(name3)
         self.assertTrue(basename3.endswith("foo.txt"))
 
+        fd1.close()
+        fd2.close()
+        fd3.close()
+
 
 try:
     file_type = file
@@ -255,28 +224,22 @@ class UniqueLineageNameTest(test_util.TempDirTestCase):
         f, path = self._call("wow")
         self.assertTrue(isinstance(f, file_type))
         self.assertEqual(os.path.join(self.tempdir, "wow.conf"), path)
+        f.close()
 
     def test_multiple(self):
+        items = []
         for _ in six.moves.range(10):
-            f, name = self._call("wow")
+            items.append(self._call("wow"))
+        f, name = items[-1]
         self.assertTrue(isinstance(f, file_type))
         self.assertTrue(isinstance(name, six.string_types))
         self.assertTrue("wow-0009.conf" in name)
+        for f, _ in items:
+            f.close()
 
-    @mock.patch("certbot.util.os.fdopen")
-    def test_failure(self, mock_fdopen):
-        err = OSError("whoops")
-        err.errno = errno.EIO
-        mock_fdopen.side_effect = err
-        self.assertRaises(OSError, self._call, "wow")
-
-    @mock.patch("certbot.util.os.fdopen")
-    def test_subsequent_failure(self, mock_fdopen):
-        self._call("wow")
-        err = OSError("whoops")
-        err.errno = errno.EIO
-        mock_fdopen.side_effect = err
-        self.assertRaises(OSError, self._call, "wow")
+    def test_failure(self):
+        with mock.patch("certbot.compat.filesystem.open", side_effect=OSError(errno.EIO)):
+            self.assertRaises(OSError, self._call, "wow")
 
 
 class SafelyRemoveTest(test_util.TempDirTestCase):
@@ -302,10 +265,10 @@ class SafelyRemoveTest(test_util.TempDirTestCase):
         # no error, yay!
         self.assertFalse(os.path.exists(self.path))
 
-    @mock.patch("certbot.util.os.remove")
-    def test_other_error_passthrough(self, mock_remove):
-        mock_remove.side_effect = OSError
-        self.assertRaises(OSError, self._call)
+    def test_other_error_passthrough(self):
+        with mock.patch("certbot.util.os.remove") as mock_remove:
+            mock_remove.side_effect = OSError
+            self.assertRaises(OSError, self._call)
 
 
 class SafeEmailTest(unittest.TestCase):
@@ -341,29 +304,28 @@ class AddDeprecatedArgumentTest(unittest.TestCase):
 
     def _call(self, argument_name, nargs):
         from certbot.util import add_deprecated_argument
-
         add_deprecated_argument(self.parser.add_argument, argument_name, nargs)
 
     def test_warning_no_arg(self):
         self._call("--old-option", 0)
-        stderr = self._get_argparse_warnings(["--old-option"])
-        self.assertTrue("--old-option is deprecated" in stderr)
+        with mock.patch("certbot.util.logger.warning") as mock_warn:
+            self.parser.parse_args(["--old-option"])
+        self.assertEqual(mock_warn.call_count, 1)
+        self.assertTrue("is deprecated" in mock_warn.call_args[0][0])
+        self.assertEqual("--old-option", mock_warn.call_args[0][1])
 
     def test_warning_with_arg(self):
         self._call("--old-option", 1)
-        stderr = self._get_argparse_warnings(["--old-option", "42"])
-        self.assertTrue("--old-option is deprecated" in stderr)
-
-    def _get_argparse_warnings(self, args):
-        stderr = six.StringIO()
-        with mock.patch("certbot.util.sys.stderr", new=stderr):
-            self.parser.parse_args(args)
-        return stderr.getvalue()
+        with mock.patch("certbot.util.logger.warning") as mock_warn:
+            self.parser.parse_args(["--old-option", "42"])
+        self.assertEqual(mock_warn.call_count, 1)
+        self.assertTrue("is deprecated" in mock_warn.call_args[0][0])
+        self.assertEqual("--old-option", mock_warn.call_args[0][1])
 
     def test_help(self):
         self._call("--old-option", 2)
         stdout = six.StringIO()
-        with mock.patch("certbot.util.sys.stdout", new=stdout):
+        with mock.patch("sys.stdout", new=stdout):
             try:
                 self.parser.parse_args(["-h"])
             except SystemExit:
@@ -512,71 +474,92 @@ class IsWildcardDomainTest(unittest.TestCase):
 class OsInfoTest(unittest.TestCase):
     """Test OS / distribution detection"""
 
-    def test_systemd_os_release(self):
-        from certbot.util import (get_os_info, get_systemd_os_info,
-                                  get_os_info_ua)
+    @mock.patch("certbot.util.distro")
+    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
+    def test_systemd_os_release_like(self, m_distro):
+        import certbot.util as cbutil
+        m_distro.like.return_value = "first debian third"
+        id_likes = cbutil.get_systemd_os_like()
+        self.assertEqual(len(id_likes), 3)
+        self.assertTrue("debian" in id_likes)
 
-        with mock.patch('os.path.isfile', return_value=True):
-            self.assertEqual(get_os_info(
-                test_util.vector_path("os-release"))[0], 'systemdos')
-            self.assertEqual(get_os_info(
-                test_util.vector_path("os-release"))[1], '42')
-            self.assertEqual(get_systemd_os_info(os.devnull), ("", ""))
-            self.assertEqual(get_os_info_ua(
-                test_util.vector_path("os-release")), "SystemdOS")
-        with mock.patch('os.path.isfile', return_value=False):
-            self.assertEqual(get_systemd_os_info(), ("", ""))
+    @mock.patch("certbot.util.distro")
+    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
+    def test_get_os_info_ua(self, m_distro):
+        import certbot.util as cbutil
+        with mock.patch('platform.system_alias',
+                        return_value=('linux', '42', '42')):
+            m_distro.name.return_value = ""
+            m_distro.linux_distribution.return_value = ("something", "1.0", "codename")
+            cbutil.get_python_os_info(pretty=True)
+            self.assertEqual(cbutil.get_os_info_ua(),
+                            " ".join(cbutil.get_python_os_info(pretty=True)))
 
-    def test_systemd_os_release_like(self):
-        from certbot.util import get_systemd_os_like
+        m_distro.name.return_value = "whatever"
+        self.assertEqual(cbutil.get_os_info_ua(), "whatever")
 
-        with mock.patch('os.path.isfile', return_value=True):
-            id_likes = get_systemd_os_like(test_util.vector_path(
-                "os-release"))
-            self.assertEqual(len(id_likes), 3)
-            self.assertTrue("debian" in id_likes)
+    @mock.patch("certbot.util.distro")
+    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
+    def test_get_os_info(self, m_distro):
+        import certbot.util as cbutil
+        with mock.patch("platform.system") as mock_platform:
+            m_distro.linux_distribution.return_value = ("name", "version", 'x')
+            mock_platform.return_value = "linux"
+            self.assertEqual(cbutil.get_os_info(), ("name", "version"))
+
+            m_distro.linux_distribution.return_value = ("something", "else")
+            self.assertEqual(cbutil.get_os_info(), ("something", "else"))
+
+    @mock.patch("warnings.warn")
+    @mock.patch("certbot.util.distro")
+    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
+    def test_get_systemd_os_info_deprecation(self, _, mock_warn):
+        import certbot.util as cbutil
+        cbutil.get_systemd_os_info()
+        self.assertTrue(mock_warn.called)
 
     @mock.patch("certbot.util.subprocess.Popen")
     def test_non_systemd_os_info(self, popen_mock):
-        from certbot.util import (get_os_info, get_python_os_info,
-                                     get_os_info_ua)
-        with mock.patch('os.path.isfile', return_value=False):
+        import certbot.util as cbutil
+        with mock.patch('certbot.util._USE_DISTRO', False):
             with mock.patch('platform.system_alias',
                             return_value=('NonSystemD', '42', '42')):
-                self.assertEqual(get_os_info()[0], 'nonsystemd')
-                self.assertEqual(get_os_info_ua(),
-                                 " ".join(get_python_os_info()))
+                self.assertEqual(cbutil.get_python_os_info()[0], 'nonsystemd')
 
             with mock.patch('platform.system_alias',
                             return_value=('darwin', '', '')):
                 comm_mock = mock.Mock()
                 comm_attrs = {'communicate.return_value':
-                              ('42.42.42', 'error')}
-                comm_mock.configure_mock(**comm_attrs)  # pylint: disable=star-args
+                            ('42.42.42', 'error')}
+                comm_mock.configure_mock(**comm_attrs)
                 popen_mock.return_value = comm_mock
-                self.assertEqual(get_os_info()[0], 'darwin')
-                self.assertEqual(get_os_info()[1], '42.42.42')
-
-            with mock.patch('platform.system_alias',
-                            return_value=('linux', '', '')):
-                with mock.patch('platform.linux_distribution',
-                                return_value=('', '', '')):
-                    self.assertEqual(get_python_os_info(), ("linux", ""))
-
-                with mock.patch('platform.linux_distribution',
-                                return_value=('testdist', '42', '')):
-                    self.assertEqual(get_python_os_info(), ("testdist", "42"))
+                self.assertEqual(cbutil.get_python_os_info()[0], 'darwin')
+                self.assertEqual(cbutil.get_python_os_info()[1], '42.42.42')
 
             with mock.patch('platform.system_alias',
                             return_value=('freebsd', '9.3-RC3-p1', '')):
-                self.assertEqual(get_python_os_info(), ("freebsd", "9"))
+                self.assertEqual(cbutil.get_python_os_info(), ("freebsd", "9"))
 
             with mock.patch('platform.system_alias',
                             return_value=('windows', '', '')):
                 with mock.patch('platform.win32_ver',
                                 return_value=('4242', '95', '2', '')):
-                    self.assertEqual(get_python_os_info(),
-                                     ("windows", "95"))
+                    self.assertEqual(cbutil.get_python_os_info(),
+                                    ("windows", "95"))
+
+    @mock.patch("certbot.util.distro")
+    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
+    def test_python_os_info_notfound(self, m_distro):
+        import certbot.util as cbutil
+        m_distro.linux_distribution.return_value = ('', '', '')
+        self.assertEqual(cbutil.get_python_os_info()[0], "linux")
+
+    @mock.patch("certbot.util.distro")
+    @unittest.skipUnless(sys.platform.startswith("linux"), "requires Linux")
+    def test_python_os_info_custom(self, m_distro):
+        import certbot.util as cbutil
+        m_distro.linux_distribution.return_value = ('testdist', '42', '')
+        self.assertEqual(cbutil.get_python_os_info(), ("testdist", "42"))
 
 
 class AtexitRegisterTest(unittest.TestCase):
@@ -608,7 +591,7 @@ class AtexitRegisterTest(unittest.TestCase):
             self.assertTrue(mock_atexit.register.called)
             args, kwargs = mock_atexit.register.call_args
             atexit_func = args[0]
-            atexit_func(*args[1:], **kwargs)  # pylint: disable=star-args
+            atexit_func(*args[1:], **kwargs)
 
 
 if __name__ == "__main__":
